#ifndef LOGGER_H
#define LOGGER_H
// ============================================================================
// Copyright(c) All Rights Reserved
//
// Summary: Logger provides logging services
// ============================================================================

#include <string>
#include <vector>

// -------------------
//! @brief holds logging functions
class Logger {
public:
    Logger() noexcept;

    //! @brief returns the current verbose setting
    //! @return the current verbose setting
    [[nodiscard]] auto verbose() const -> bool
    {
        return m_verbose;
    }

    //! @brief sets the verbose level
    //! @param val if true writes lines to stdout, false to ignore
    void set_verbose(bool val)
    {
        m_verbose = val;
    }

    void start(const char* fmt, ...) const;

    void line(const char* fmt, ...) const;

    void ok(const char* fmt, ...) const;

    void err(const char* fmt, ...) const;

    void bug(const char* fmt, ...) const;

    void warn(const char* fmt, ...) const;

    void dbg(const char* fmt, ...) const;

    void raw(const char* fmt, ...) const;

    void output(int lineno, const char* fmt, ...) const;

private:
    //! @brief holds verbosity setting
    bool m_verbose = true;

    void _write_line(const bool print_always, const char* const tag, const char* const fmt, va_list args) const;

    // === for unit test purposes only

    //! @brief sets UT mode
    static void ut_set_mode();

    //! @brief clears the current saved lines
    void ut_clear();

    //! @brief returns the current saved lines
    //! @return saved lines
    auto ut_lines() -> std::vector<std::string>;
};

#endif // LOGGER_H
